﻿//
var BootState = {

    //
    init: function()
    {
        game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
        game.scale.fullScreenScaleMode = Phaser.ScaleManager.SHOW_ALL;       
        game.scale.pageAlignHorizontally = true;
        game.scale.pageAlignVertically = true;
        game.scale.onSizeChange.add(this.onSizeChange, this);
        game.scale.setResizeCallback(this.onResize, this);        

        if(!game.device.desktop)
        {
            game.scale.forceOrientation(true, false);
            game.scale.enterIncorrectOrientation.add(this.onEnterIncorrectOrientation, this);
            game.scale.leaveIncorrectOrientation.add(this.onLeaveIncorrectOrientation, this);
        }

        game.input.maxPointers = 1;
        game.stage.disableVisibilityChange = false;
        game.canvas.parentElement.style.backgroundColor = '#1e0d15';

        this.additionalAudioCheck();

        this.resizeGame();
    },

    preload: function()
    {
        game.load.atlas('load', 'assets/atlas_load.png', 'assets/atlas_load.json');
    },

    //
    create: function()
    {
        game.state.add('load', LoadState);
        game.state.add('home', HomeState); 
        game.state.add('play', PlayState);
        game.state.start('load');
    },

    onResize: function()
    {        
        var ww = window.innerWidth;
        var wh = window.innerHeight;        
        if(R.prevWindowWidth != ww || R.prevWindowHeight != wh) this.onSizeChange();
        
        var gameContainer = document.getElementById(game.parent);
        gameContainer.style.width = ww + "px";
        gameContainer.style.height = wh + "px";
    },

    onSizeChange: function()
    {
        if(game.device.desktop || window.innerWidth > window.innerHeight)
        {
            this.resizeGame();
            game.state.resize(R.gameWidth, R.gameHeight);
        }
    },

    resizeGame: function()
    {
        var ww = window.innerWidth;
        var wh = window.innerHeight;   

        R.prevWindowWidth = ww;
        R.prevWindowHeight = wh;
        
        R.gameWidth = R.BASE_GAME_HEIGHT / wh * ww;

        if(R.gameWidth > R.MAX_GAME_WIDTH) R.gameWidth = R.MAX_GAME_WIDTH;
        else if(R.gameWidth < R.MIN_GAME_WIDTH) R.gameWidth = R.MIN_GAME_WIDTH;

        R.halfGameWidth = R.gameWidth * 0.5;

        game.scale.setGameSize(R.gameWidth, R.BASE_GAME_HEIGHT);
    },

    onEnterIncorrectOrientation: function()
    {
        game.paused = true;
    },

    onLeaveIncorrectOrientation: function()
    {
        game.paused = false;
        this.onSizeChange();
    },

    additionalAudioCheck: function()
    {
        //second check sound
        /*
            Sharp = SH-01F
            Fujitsu = F-01F
            Xperia A = SO-04E
            Sharp Mini = SHL24
            */

        function isStock()
        {
            var matches = window.navigator.userAgent.match(/Android.*AppleWebKit\/([\d.]+)/);
            return matches && matches[1] < 537;
        }

        var ua = navigator.userAgent; // Returns a string which tells you what device you're using
        var isSharpStock = ((/SHL24|SH-01F/i).test(ua)) && isStock(); // Checks if device is, Sharp(SH-01F) or Sharp Mini(SHL24)
        var isXperiaAStock = ((/SO-04E/i).test(ua)) && isStock(); // Checks if device is, Xperia A(SO-04E)
        var isFujitsuStock = ((/F-01F/i).test(ua)) && isStock(); // Checks if device is, Fujitsu(F-01F)

        if(isSharpStock || isXperiaAStock || isFujitsuStock) game.device.webAudio = false;
    }    
};