﻿var R = R || {};

R.groundLocY = 584;

R.maxCountWidth = 17;
R.maxCountHeight = 10;

R.furtherNextPillar = 10000;

R.intensityShake = 4;

//
R.BoxContainer = function()
{ 
    Phaser.Group.call(this, game);
    
    var quantity = R.maxCountWidth * R.maxCountHeight

    for(var i = 0; i < quantity; i++)
    {
        var box = new R.Box();
        this.add(box);
    }
    
    this.pillars = [];   
    this.tailPillar = null;

    this.allowNextPillar = false;

    this.firstY = R.groundLocY - R.boxBodySize * 0.5;  
    
    this.tweenMove = null;
    this.isItEnd = false;

    this.pillarsExist = 1;

    //boom
    this.booms = game.add.group();
    quantity = Math.floor(quantity * 0.2);    
    var frameNames = [];
    for(var i = 6; i > 0; i--) frameNames.push('boom' + i.toString());
    for(var i = 0; i < quantity; i++)
    {
        var b = this.booms.create(0, 0, 'play', 'boom1', false);
        b.anchor.set(0.5);
        b.animations.add('b', frameNames, 18, false).onComplete.add(b.kill, b);
    }

    //    
    this.timer = game.time.add(new Phaser.Timer(game, false));    
    this.timer.start();

    //    
    this.createPillar();
    this.tailPillar.x = R.boxBodySize * 0.5;
    this.tailPillar.updateShift();

    game.time.events.loop(1000, this.checkAllBoxesForKindNeighbour, this);
};

R.BoxContainer.prototype = Object.create(Phaser.Group.prototype);
R.BoxContainer.prototype.constructor = R.BoxContainer;

R.BoxContainer.prototype.createPillar = function()
{
    var i = R.maxCountHeight;
    var tail = this.tailPillar;
    var box = null;   
    var x = -R.boxBodySize * 0.5;
    var y = this.firstY;
    var base = null;
    var prev = null;    

    while(i--)
    {        
        box = this.getFirstExists(false, false, x, y, 'play', 'box10');       
        if(this.tailPillar) this.bringToTop(box);
        box.setup(true);

        if(prev)
        {
            box.boxBottom = prev;
            prev.boxTop = box;
        }
        prev = box;

        if(tail)
        {
            box.boxRight = tail;
            tail.boxLeft = box;
            tail = tail.boxTop;
        }
              
        if(!base) base = box;        
        box.boxBase = base;

        y -= R.boxBodySize;
    }      
    
    this.pillars.push(base);
    this.tailPillar = base;  
};

R.BoxContainer.prototype.createAdditionallyBlocks = function()
{
    var i = this.pillars.length;
    while(i--) this.pillars[i].addBlock();   
};

R.BoxContainer.prototype.addPillar = function()
{    
    this.createAdditionallyBlocks();
    this.createPillar();    

    this.tweenMove = game.add.tween(this.tailPillar).to({ x: this.tailPillar.x + R.boxBodySize }, 700, Phaser.Easing.Linear.None, true);
    this.tweenMove.onComplete.add(this.next, this);
    this.tweenMove.onUpdateCallback(this.updateShiftPillar, this);   

    if(R.sfx.move_stone) R.sfx.move_stone.play();
   
    this.allowNextPillar = false;   
    ++this.pillarsExist;
    
    PlayState.dustyRain.fall();    
};

R.BoxContainer.prototype.next = function()
{    
    game.camera.y = 0;
    this.tweenMove = null;
    this.timer.removeAll();    
    
    if(this.pillarsExist < R.maxCountWidth) this.timer.add(R.furtherNextPillar, function() { this.allowNextPillar = true; }, this);   

    if(this.tailPillar) this.tailPillar.updateShift();

    if(R.sfx.move_stone && R.sfx.move_stone.isPlaying) R.sfx.move_stone.stop();

    PlayState.increaseDepth();
};

R.BoxContainer.prototype.updateShiftPillar = function()
{
    this.tailPillar.updateShift();
    game.camera.y = game.rnd.between(-R.intensityShake, R.intensityShake);
};

R.BoxContainer.prototype.pick = function(x, y)
{
    x -= this.parent.x;    
    
    var base = null;
    var box = null;
    var hs = R.boxBodySize / 2;

    var i = this.pillars.length;
    while(i--)
    {
        base = this.pillars[i];
        
        if(x > (base.x - hs) && x < (base.x + hs))
        {
            box = base;
            do
            {
                if(y > (box.y - hs) && y < (box.y + hs) && box.kind > 0) return box;
                box = box.boxTop;
            }
            while(box);

            return null;
        }
    }

    return null;
};

R.BoxContainer.prototype.update = function()
{
    if(this.allowNextPillar) this.addPillar();
    else
    {
        var i = this.children.length;
        while(i--) this.children[i].update();        
    }
};

R.BoxContainer.prototype.pause = function(isPuased)
{
    if(isPuased)
    {
        if(this.tweenMove)
        {
            this.tweenMove.pause();
            if(R.sfx.move_stone) R.sfx.move_stone.pause();
        }
        else this.timer.pause();
    }
    else
    {
        if(this.tweenMove)
        {
            this.tweenMove.resume();
            if(R.sfx.move_stone) R.sfx.move_stone.resume();
        }
        else this.timer.resume();
    }
};

R.BoxContainer.prototype.forcedNext = function()
{
    if(this.tweenMove || this.pillarsExist === R.maxCountWidth) return;
    this.timer.removeAll();
    this.addPillar();
};

R.BoxContainer.prototype.checkPillarsCollapse = function()
{
    var p = null;
    var i = this.pillars.length;
    while(i--)
    {
        p = this.pillars[i];
        if(p.isPillarEmpty())
        {
            p.connectNeighbors();            
            this.pillars.splice(i, 1);
            --this.pillarsExist;
            if(this.tailPillar === p) this.tailPillar = this.pillarsExist > 0 ? this.pillars[this.pillarsExist - 1] : null;
        }
    }

    if(this.tailPillar === null && this.tweenMove) this.tweenMove.stop(true);    
};

R.BoxContainer.prototype.makeBoom = function(x, y)
{
    var b = this.booms.getFirstExists(false, false, x, y, 'play', 'boom1');
    if(b) b.animations.play('b').speed = game.rnd.between(16, 25);    
    if(R.sfx.axe_stone_crushed && !R.sfx.axe_stone_crushed.isPlaying) R.sfx.axe_stone_crushed.play("", 0, 1, false);
};

R.BoxContainer.prototype.stop = function()
{
    this.timer.removeAll();
};

R.BoxContainer.prototype.getFreePlaces = function(outX)
{    
    var x0 = this.pillars.length > 0 ? this.pillars[0].x : 0;
    var x1 = R.maxCountWidth * R.boxBodySize;   
    var x = x0 + R.boxBodySize;

    while(x < x1)
    {
        outX.push(x);
        x += R.boxBodySize;
    }    
   
    for(var i = 0; i < this.pillars.length; ++i)
    {
        x = this.pillars[i].x - R.boxBodySize - 0.5;
        x1 = ((i + 1) < this.pillars.length ? this.pillars[i + 1].x : 0);
        while(x > x1)
        {
            outX.unshift(x);
            x -= R.boxBodySize;
        }
    }
};

R.BoxContainer.prototype.checkAllBoxesForKindNeighbour = function()
{   
    var i = this.children.length;
    while(i--) if(this.children[i].checkForSame()) return;
    this.forcedNext();
};