﻿var R = R || {};

R.boxSpriteSize = 62;
R.boxBodySize = 53;

R.boxAnchor = (R.boxSpriteSize - R.boxBodySize + R.boxBodySize * 0.5) / R.boxSpriteSize;
R.boxBodyOffset = Math.floor((R.boxSpriteSize - R.boxBodySize) * R.boxAnchor);

R.boxGravity = 0.5;

R.boxCollapsed = 0;

//Box
R.Box = function()
{
    Phaser.Sprite.call(this, game, 0, 0, 'play', 'box10');
    this.anchor.set(R.boxAnchor);   

    this.kind = 0;
    this.subkind = 0;

    this.boxLeft = null;
    this.boxTop = null;
    this.boxRight = null;
    this.boxBottom = null;
    this.boxBase = null;

    this.placeY = 0;
    this.vy = 0;

    this.exists = false;
};

R.Box.prototype = Object.create(Phaser.Sprite.prototype);
R.Box.prototype.constructor = R.Box;

R.Box.prototype.setup = function(setToZero)
{
    if(setToZero)
    {
        this.boxLeft = null;
        this.boxTop = null;
        this.boxRight = null;
        this.boxBottom = null;

        this.placeY = this.y;
        this.update = this.sleep;        
    }

    this.vy = 0;

    this.kind = game.rnd.between(1, 4);

    this.subkind = 0;
    if(this.kind === 1) { if(game.rnd.between(0, 100) > 85) this.subkind = game.rnd.between(1, 3); }
    else if(this.kind === 2) this.subkind = game.rnd.between(0, 1);

    this.frameName = 'box' + this.kind.toString() + this.subkind.toString();
};

R.Box.prototype.adjustPillarX = function()
{
    var boxTop = this.boxTop;
    while(boxTop)
    {
        boxTop.x = this.x;
        boxTop = boxTop.boxTop;
    }
};

R.Box.prototype.updateShift = function()
{
    var boxRight = this.boxRight;
    var x = this.x;
    this.adjustPillarX();

    while(boxRight && (boxRight.x - x) < R.boxBodySize)
    {
        boxRight.x = x + R.boxBodySize;
        boxRight.adjustPillarX();
        x = boxRight.x;        
        boxRight = boxRight.boxRight;
    }   
};

R.Box.prototype.check = function()
{
    var kind = this.kind;

    R.boxCollapsed = 0;

    if((this.boxLeft && this.boxLeft.kind === kind && (this.x - this.boxLeft.x - 0.5) < R.boxBodySize) ||
       (this.boxTop && this.boxTop.kind === kind) ||
       (this.boxRight && this.boxRight.kind === kind && (this.boxRight.x - this.x - 0.5) < R.boxBodySize) ||
       (this.boxBottom && this.boxBottom.kind === kind))
    {
        this.collapse(kind);
    }
    
    if(R.boxCollapsed > 0) PlayState.scoreCollapse(this.x, this.y, (kind === 2 ? R.boxCollapsed * R.boxCollapsed * 10 : R.boxCollapsed), 100);    

    this.parent.checkPillarsCollapse();    
};

R.Box.prototype.collapse = function(kind)
{
    if(this.kind === 0 || this.update !== this.sleep) return;

    this.visible = false;

    if(kind === 2) ++R.boxCollapsed;
    else if(this.subkind > 0) R.boxCollapsed += 100;
    
    this.kind = 0;
    this.update = this.sleep;
    this.parent.makeBoom(this.x, this.y);
    
    if(this.boxTop)
    {
        if(this.boxTop.kind === kind) this.boxTop.collapse(kind);
        else if(this.boxTop.kind !== 0) this.boxTop.update = this.boxTop.sniff;
    }
    if(this.boxLeft && this.boxLeft.kind === kind && (this.x - this.boxLeft.x - 0.5) < R.boxBodySize) this.boxLeft.collapse(kind);
    if(this.boxRight && this.boxRight.kind === kind && (this.boxRight.x - this.x - 0.5) < R.boxBodySize) this.boxRight.collapse(kind);
    if(this.boxBottom && this.boxBottom.kind === kind) this.boxBottom.collapse(kind);    
};

R.Box.prototype.passDown = function(bb)
{
    bb.frameName = this.frameName;
    bb.kind = this.kind;
    bb.subkind = this.subkind;
    bb.y = this.y;
    bb.visible = true;
    bb.update = bb.fall;
    bb.vy = this.vy;    
    this.visible = false;
    this.kind = 0;
    this.update = this.sleep;    
};

R.Box.prototype.sleep = function() { };

R.Box.prototype.sniff = function()
{
    var bb = this.boxBottom;
    if(bb && bb.kind === 0)
    {
        this.passDown(bb);
        if(this.boxTop && this.boxTop.kind !== 0) this.boxTop.update = this.boxTop.sniff;
    }
};

R.Box.prototype.fall = function(boundY)
{
    this.vy += R.boxGravity;
    this.y += this.vy;

    if(this.y > this.placeY)
    {
        if(this.boxBottom && this.boxBottom.kind === 0)
        {
            this.passDown(this.boxBottom);
            if(this.boxTop && this.boxTop.update === this.boxTop.sleep && this.boxTop.kind !== 0) this.boxTop.update = this.boxTop.sniff;
        }
        else
        {
            this.y = this.placeY;
            this.vy = 0;
            this.update = this.sleep;
            if(R.sfx.stone_on_stone && !R.sfx.stone_on_stone.isPlaying) R.sfx.stone_on_stone.play();
        }
    }
};

R.Box.prototype.isPillarEmpty = function()
{
    var box = this;
    while(box)
    {
        if(box.visible) return false;
        box = box.boxTop;
    }
    return true;
};

R.Box.prototype.connectNeighbors = function()
{    
    var box = this;
    while(box)
    {
        if(box.boxLeft) box.boxLeft.boxRight = box.boxRight;
        if(box.boxRight) box.boxRight.boxLeft = box.boxLeft;
        box.kill();
        box.update = box.sleep;
        box = box.boxTop;
    }
};

R.Box.prototype.addBlock = function()
{
    var c = 1;
    var top = this.boxTop;
    while(true)
    {
        if(top.kind !== 0) ++c;
        if(c > 6) return;
        if(top.boxTop) top = top.boxTop;
        else break;
    }

    if(game.rnd.between(0, 100) < 70) return;

    top.setup(false);
    top.y = 10;
    top.update = top.fall;
};

R.Box.prototype.checkForSame = function()
{    
    if(this.kind === 0) return false;
    if(this.update !== this.sleep) return true;

    if(this.boxLeft && this.boxLeft.kind === this.kind && ((this.x - this.boxLeft.x - 0.5) < R.boxBodySize)) return true;
    if(this.boxRight && this.boxRight.kind === this.kind && ((this.boxRight.x - this.x - 0.5) < R.boxBodySize)) return true;
    if(this.boxTop && this.boxTop.kind === this.kind) return true;
    if(this.boxBottom && this.boxBottom.kind === this.kind) return true;

    return false;
};