﻿var R = R || {};

//GUI
R.createText = function(x, y, size, text, fill, isStroke, strokeThicknes, isWrap)
{
    if(!fill) fill = '#ffffff';
    var style = { font: R.fontName, fontSize: size, fill: fill, fontWeight: '100', align: 'center' };
    var label = game.add.text(x, y, text, style);   
    label.anchor.set(0.5);
    if(isStroke)
    {
        label.stroke = '#000000';
        label.strokeThickness = strokeThicknes;
    }
    if(isWrap)
    {
        label.wordWrap = true;
        label.wordWrapWidth = game.width;
    }
    return label;
};

//Button
R.createButton = function(x, y, callback, context, frame, framePushed)
{    
    var button = game.add.button(x, y, 'gui', callback, context, frame, frame, framePushed, frame);
    button.anchor.set(0.5);   
    if(R.sfx.button) button.setDownSound(R.sfx.button);
    return button;
};

//SoundButton
R.SoundButton = function(x, y, frameOn, frameOff)
{
    Phaser.Button.call(this, game, x, y, 'gui', this.onClick, this, frameOn, frameOn, 'btn_sound_off_pushed', frameOn);

    this.anchor.set(0.5);

    this.frameOn = frameOn;
    this.frameOff = frameOff;

    this.checkFrame();

    if(R.sfx.button) this.setDownSound(R.sfx.button);

    this.exists = game.device.webAudio;   
};

R.SoundButton.prototype = Object.create(Phaser.Button.prototype);
R.SoundButton.prototype.constructor = R.SoundButton;

R.SoundButton.prototype.checkFrame = function()
{
    var frame = game.sound.mute ? this.frameOff : this.frameOn;
    this.setFrames(frame, frame, 'btn_sound_off_pushed', frame);
};

R.SoundButton.prototype.onClick = function()
{    
    game.sound.mute = !game.sound.mute;
    this.checkFrame();
};

//OptionsScreen
R.OptionsScreen = function()
{
    Phaser.Sprite.call(this, game, 0, 0, null);

    this.curtain = game.add.sprite(0, 0, R.bitmapData);        
    this.addChild(this.curtain);

    this.buttons = game.add.group();
    this.addChild(this.buttons);

    this.homeButton = R.createButton(0, 0, this.onHomeButton, this, 'btn_home', 'btn_home_pushed');
    this.buttons.add(this.homeButton);

    this.retryButton = R.createButton(150, 0, this.onRetryButton, this, 'btn_retry', 'btn_retry_pushed');
    this.buttons.add(this.retryButton);

    this.soundButton = new R.SoundButton(0, 150, 'btn_sound_on', 'btn_sound_off');
    this.buttons.add(this.soundButton);

    this.continueButton = R.createButton(this.soundButton.exists ? 150 : 75, 150, this.onContinueButton, this, 'btn_play', 'btn_play_pushed');
    this.buttons.add(this.continueButton);

    this.tutorialButton = R.createButton(game.width - 100, R.halfGameHeight + 20, this.onTutorialButton, this, 'btn_tutorial', 'btn_tutorial_pushed');
    this.buttons.add(this.tutorialButton);

    this.exists = false;
    this.enableButtons(false);
};

R.OptionsScreen.prototype = Object.create(Phaser.Sprite.prototype);
R.OptionsScreen.prototype.constructor = R.OptionsScreen;

R.OptionsScreen.prototype.resize = function(width, height)
{
    var w = (this.retryButton.x + this.retryButton.width * 0.5) - (this.homeButton.x - this.homeButton.width * 0.5);
    this.x = (width - w + this.homeButton.width) * 0.5;
    this.curtain.x = -this.x;
    this.curtain.scale.set(game.width / R.bmdSize, game.height / R.bmdSize);
    this.tutorialButton.x = R.halfGameWidth + 20;
};

R.OptionsScreen.prototype.enableButtons = function(enable)
{
    this.homeButton.inputEnabled = enable;
    this.retryButton.inputEnabled = enable;
    this.soundButton.inputEnabled = enable;
    this.continueButton.inputEnabled = enable;
    this.tutorialButton.inputEnabled = enable;
};

R.OptionsScreen.prototype.open = function()
{
    this.soundButton.checkFrame();
    game.world.add(this);
    this.exists = true;

    this.curtain.alpha = 0.0;
    game.add.tween(this.curtain).to({ alpha: 0.66 }, 200, Phaser.Easing.Linear.None, true);

    var bhh = this.homeButton.height * 0.5;
    this.buttons.y = game.height + bhh;
    game.add.tween(this.buttons).to({ y: (game.height - 248) * 0.5 + bhh }, 300, Phaser.Easing.Back.Out, true).onComplete.add(this.enableButtons, this, true);
    
    this.resize(game.width, game.height);
};

R.OptionsScreen.prototype.onHomeButton = function()
{
    this.enableButtons(false);
    R.sceneTransition(200, 'home');

    Publisher.showAdvertising();
};

R.OptionsScreen.prototype.onContinueButton = function()
{
    this.enableButtons(false);
    game.add.tween(this.buttons).to({ y: game.height + this.homeButton.height * 0.5 }, 300, Phaser.Easing.Back.In, true).onComplete.add(this.enableButtons, this, true);
    game.add.tween(this.curtain).to({ alpha: 0.0 }, 200, Phaser.Easing.Linear.None, true, 300).onComplete.add(this.endClose, this);
};

R.OptionsScreen.prototype.onRetryButton = function()
{
    R.level = 1;
    R.score = 0;
    R.depth = 0;
    R.purposeDepth = 18;

    this.enableButtons(false);
    R.sceneTransition(200, 'play');
};

R.OptionsScreen.prototype.onTutorialButton = function()
{
    this.enableButtons(false);
    this.exists = false;
    game.world.remove(this);
    PlayState.showTutorial();
};

R.OptionsScreen.prototype.endClose = function()
{    
    this.exists = false;
    game.world.remove(this);
    PlayState.onClosePauseScreen();
};

//EndScreen
R.EndScreen = function(isWin, delay)
{
    Phaser.Sprite.call(this, game, 0, 0, null);

    this.curtain = game.add.sprite(0, (game.height - 200) * 0.5, R.bitmapData);
    this.curtain.scale.set(game.width / R.bmdSize, 200 / R.bmdSize);
    this.curtain.alpha = 0.66;
    this.addChild(this.curtain);

    this.label = R.createText(0, 250, 46, isWin ? R.strings.complete : R.strings.game_over, '#ffffff');
    this.addChild(this.label);

    this.continueButton = R.createButton(0, 350, this.onContinueButton, this, 'btn_play', 'btn_play_pushed');    
    this.addChild(this.continueButton); 

    this.homeButton = R.createButton(0, this.continueButton.y, this.onHomeButton, this, 'btn_home', 'btn_home_pushed');
    this.addChild(this.homeButton);
   
    this.isWin = isWin;

    //    
    this.enableButtons(false);
    game.world.add(this);
    this.resize(game.width, game.height);

    this.y = game.height;
    game.add.tween(this).to({ y: 0 }, 1000, Phaser.Easing.Back.Out, true, delay).onComplete.add(function() { this.enableButtons(true); }, this);
};

R.EndScreen.prototype = Object.create(Phaser.Sprite.prototype);
R.EndScreen.prototype.constructor = R.EndScreen;

R.EndScreen.prototype.resize = function(width, height)
{    
    this.curtain.scale.x = width / R.bmdSize;

    this.label.x = R.halfGameWidth;
    this.homeButton.x = R.halfGameWidth - 80;
    this.continueButton.x = R.halfGameWidth + 80;
};

R.EndScreen.prototype.enableButtons = function(enabled)
{
    this.continueButton.inputEnabled = enabled;
    this.homeButton.inputEnabled = enabled;
};

R.EndScreen.prototype.onContinueButton = function()
{
    R.saveGame();

    if(this.isWin)
    {
        ++R.level;        
        R.purposeDepth += 4;
    }
    else
    {
        R.level = 1;        
        R.purposeDepth = 18;
        R.score = 0;
    }

    R.depth = 0;

    this.enableButtons(false);
    R.sceneTransition(200, 'play');

    Publisher.showAdvertising();
};

R.EndScreen.prototype.onHomeButton = function()
{
    R.saveGame();

    this.enableButtons(false);
    R.sceneTransition(200, 'home');

    Publisher.showAdvertising();
};

//Background
R.Background = function(textureName, frameName)
{
    Phaser.Group.call(this, game);

    var frame = game.cache.getFrameData(textureName).getFrameByName(frameName);

    this.textureName = textureName;
    this.frameName = frameName;

    this.tileWidth = frame.width;

    this.fillArea(R.gameWidth, R.gameHeight);
};

R.Background.prototype = Object.create(Phaser.Group.prototype);
R.Background.prototype.constructor = R.Background;

R.Background.prototype.fillArea = function(areaWidth)
{
    var cols = Math.round(areaWidth / this.tileWidth + 0.5);

    var x = 0;
    var y = 0;

    for(var c = 0; c < cols; c++)
    {
        this.create(x, y, this.textureName, this.frameName);
        x += this.tileWidth;
    }
};

R.Background.prototype.resize = function(width)
{
    this.removeAll(true, false);
    this.fillArea(width);
};

//Backround stuff
R.BackgroundStuff = function()
{
    this.objects = [];
};

R.BackgroundStuff.prototype = {

    constructor: R.BackgroundStuff,

    dotting: function(x0)
    {        
        var x = 0;
        var o = null;

        for(var i in this.objects) this.objects[i].destroy();

        //stones3
        x = x0 - 10;
        var f = 1;
        while(true)
        {
            o = game.add.sprite(x, game.height - (f === 1 ? 120 : 122), 'play', f === 1 ? 'stones3' : 'stones4');
            this.objects.push(o);
            x += o.width + game.rnd.between(o.width * 0.25, o.width * 0.5);
            if(x > game.width) break;
            f = 1 - f;
        }

        //stones1
        x = x0 + game.rnd.between(50, 74);
        while(true)
        {            
            o = game.add.sprite(x, game.height - 101, 'play', 'stones1');
            this.objects.push(o);
            x += 148 + game.rnd.between(50, 74);
            if(x > game.width) break;
        }

        //stones2
        x = x0 + game.rnd.between(20, 50);
        while(true)
        {
            o = game.add.sprite(x, game.height - game.rnd.between(15, 20), 'play', 'stones2');
            o.anchor.set(0.5);
            o.rotation = game.rnd.realInRange(0, Math.PI * 2);
            this.objects.push(o);
            x += 54 + game.rnd.between(20, 50);
            if(x > game.width) break;
        }

        //branch
        o = game.add.sprite(game.rnd.between(x0 + 50, game.width), game.height - 62, 'play', 'branch');
        o.anchor.set(0.5, 1.0);
        this.objects.push(o);

        //crystal
        o = game.add.sprite(game.rnd.between(x0 + 50, game.width), game.height - 63, 'play', 'crystall_floor_blue');
        o.anchor.set(0.5, 1.0);
        this.objects.push(o);

        //coin
        o = game.add.sprite(game.rnd.between(x0 + 12, game.width), game.height - 61, 'play', 'coin');
        o.anchor.set(0.5, 1.0);
        this.objects.push(o);

        //stalagmite
        x = x0 + game.rnd.between(40, 60);
        f = game.rnd.between(1, 3);
        while(true)
        {
            o = game.add.sprite(x, 40, 'play', 'stalagmite' + f.toString());
            o.anchor.x = 0.5;
            this.objects.push(o);            
            x += o.width + game.rnd.between(50, 200);
            if(x > game.width) break;
            if(++f > 3) f = 1;
        }
    }
};

//FlyScore
R.FlyScore = function(size, fill)
{
    Phaser.Text.call(this, game, 0, 0, '');

    this.anchor.set(0.5);
    this.font = R.fontName;    
    this.align = 'center';
    this.fontSize = 40;
    this.fill = '#e017cb';
    this.fontWeight = '100';
    this.exists = false;

    this.score = 0;
};

R.FlyScore.prototype = Object.create(Phaser.Text.prototype);
R.FlyScore.prototype.constructor = R.FlyScore;

R.FlyScore.prototype.setup = function(x, y, xt, yt, score, delay)
{
    this.position.set(x, y);
    this.score = score;
    this.text = score.toString();
    this.exists = true;    
    
    var distance = Phaser.Math.distance(x, y, xt, yt);
    var duration = distance * 1.2;

    game.add.tween(this).to({ x: xt, y: yt }, duration, Phaser.Easing.Quadratic.Out, true, delay).onComplete.add(PlayState.endFlyScore, PlayState, this);
};